@@ -1,6 +1,6 @@
 /*
  * report.c
- * Copyright 2007-2008 William Pitcock and Matti Hämäläinen
+ * Copyright 2007-2008 Ariadne Conill and Matti Hämäläinen
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -69,7 +69,7 @@ void audtool_whine_tuple_fields (void)
     if (! fields)
         exit (1);
 
-    audtool_whine ("Field names include, but are not limited to:\n");
+    audtool_whine ("Field names include:\n");
 
     char * * tmp = fields;
     int col = 0;
