@@ -17,8 +17,6 @@
  * the use of this software.
  */
 
-#include <stdlib.h>
-
 #include <QApplication>
 #include <QLibraryInfo>
 #include <QProxyStyle>
@@ -114,6 +112,29 @@ EXPORT void init()
         return;
 
     aud_config_set_defaults("audqt", audqt_defaults);
+    log_init();
+
+    // The QApplication instance is created only once and is not deleted
+    // by audqt::cleanup(). If it already exists, we are done here.
+    if (qApp)
+        return;
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0) && defined(_WIN32)
+    QApplication::setHighDpiScaleFactorRoundingPolicy(
+        Qt::HighDpiScaleFactorRoundingPolicy::Floor);
+#endif
+
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
+    // Use X11/XWayland by default, but allow to overwrite it.
+    // Especially the Winamp interface is not usable yet on Wayland
+    // due to limitations regarding application-side window positioning.
+    auto platform = qgetenv("QT_QPA_PLATFORM");
+    if (platform.isEmpty() && qEnvironmentVariableIsSet("DISPLAY"))
+        qputenv("QT_QPA_PLATFORM", "xcb");
+    else if (platform != "xcb")
+        AUDWARN("X11/XWayland was not detected. This is unsupported, "
+                "please do not report bugs.\n");
+#endif
 
     static char app_name[] = "audacious";
     static int dummy_argc = 1;
@@ -165,14 +186,8 @@ EXPORT void init()
     QApplication::setFont(QApplication::font("QSmallFont"), "QTreeView");
     QApplication::setFont(QApplication::font("QTipLabel"), "QStatusBar");
 #endif
-
-    log_init();
 }
 
-EXPORT void run() { qApp->exec(); }
-
-EXPORT void quit() { qApp->quit(); }
-
 EXPORT void cleanup()
 {
     if (--init_count)
@@ -187,12 +202,9 @@ EXPORT void cleanup()
 
     log_cleanup();
 
-    delete qApp;
-}
-
-EXPORT QIcon get_icon(const char * name)
-{
-    return QIcon::fromTheme(name);
+    // We do not delete the QApplication here due to issues that arise
+    // if it is deleted and then re-created. Instead, it is deleted
+    // later during shutdown; see mainloop_cleanup() in libaudcore.
 }
 
 EXPORT QGradientStops dark_bg_gradient(const QColor & base)
@@ -250,9 +262,9 @@ EXPORT QVBoxLayout * make_vbox(QWidget * parent, int spacing)
 
 EXPORT void setup_proxy_style(QProxyStyle * style)
 {
-    // set the correct base style ("fusion" or native)
+    // set the correct base style (dark or native)
     if (!strcmp(aud_get_str("audqt", "theme"), "dark"))
-        style->setBaseStyle(QStyleFactory::create("fusion"));
+        style->setBaseStyle(create_dark_style());
     else
         style->setBaseStyle(nullptr);
 
